@@ -91,8 +91,8 @@ StatusBar::StatusBar( QWidget *parent, const char *name )
 
     //b1->setIconSet( SmallIconSet( "cancel" ) );
     //b2->setIconSet( SmallIconSet( "2uparrow") );
-    b1->setIcon( QPixmap( Global::instance()->pathPic() + "pic/messagebox_critical.png" ));
-    b2->setIcon( QPixmap( Global::instance()->pathPic() + "pic/messagebox_info.png" ));
+    b1->setIcon( QIcon( ":/pic/messagebox_critical.png" ));
+    b2->setIcon( QIcon( ":/pic/messagebox_info.png" ));
     b2->setCheckable( true );
 //     QToolTip::add( b1, tr( "Abort all background-operations" ) );
 //     QToolTip::add( b2, tr( "Show progress detail" ) );
